/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>

#include <sys/socket.h>
#include <netinet/in.h>

#include <alert.h>
#include <signature.h>
#include <idx.h>
#include <matcher.h>
#include <plugin.h>
#include <decode.h>
#include <args.h>
#include <detect.h>

struct alert *cur_alert;
unsigned int alert_depth;
unsigned int cur_depth;

struct generator ipx_gen=init_generator("sig.ipx", NULL);
	
struct alert *cur_alert;
unsigned int alert_depth;
unsigned int cur_depth;

static struct sub_table ipx_subs[]={
	{"src", "ipx_src"},
	{"dst", "ipx_dst"},
	{"sport", "ipx_srcsock"},
	{"dport", "ipx_dstsock"},
	{NULL, NULL},
};

/* IPX: Internetwork Packet eXchange */
int ipx_print(struct packet *p, int l, char *buf, size_t buflen);
void ipx_decode(struct packet *);
void ipx_match(struct packet *p, int l);
int ipx_commit(struct rule *r);
struct proto ipx_p={
	.name="ipx",
	.decode=ipx_decode,
	.print=ipx_print,
	.sig_add=ipx_commit,
	.sig_match=ipx_match,
	.sig_subs = ipx_subs,
};
struct proto_req ipx_r[]={
        proto_request("ethernet", __constant_htons(0x8137)),
        proto_request("sll", __constant_htons(0x8137)),
        proto_request("linux", __constant_htons(0x8137)),
        proto_request("802.3-novell", 0),
        proto_request("llc", 0xe0),
	null_request()
};

static const char *pnames[]={
	"SAP/NSP/Hello", "RIP", "Echo", "Error",
	"PEP", "SPX",
};

int ipx_print(struct packet *p, int l, char *buf, size_t buflen)
{
	struct layer *a = &p->layer[l];
	const char *ptype="Unknown";
	char *b=buf;
	int c;
	char *x;

	switch (a->h.ipx->type) {
		case 0 ... 5:
			ptype=pnames[a->h.ipx->type];
			break;
		case 17:
			ptype="NCP";
			break;
		case 20:
			ptype="NetBIOS";
			break;
		case 104:
			ptype="NDS NCP";
			break;
	};
	x=a->h.ipx->src.node;
	for(c=0; c<6; c++, x++)
		b+=sprintf(b, "%02x%c", *x&0xFF, c==5 ? ' ' : ':');
	b+=sprintf(b,"> ");
	x=a->h.ipx->dst.node;
	for(c=0; c<6; c++, x++)
		b+=sprintf(b, "%02x%c", *x&0xFF, c==5 ? ' ' : ':');
	b+=sprintf(b,"type=0x%.2x(%s) net=0x%.8x:0x%.8x skt=0x%.4x:0x%.4x ",
				a->h.ipx->type,
				ptype,
				ntohl(a->h.ipx->src.net),
				ntohl(a->h.ipx->dst.net),
				ntohs(a->h.ipx->src.sock),
				ntohs(a->h.ipx->dst.sock));
	return 0;
}

void ipx_decode(struct packet *p)
{
	struct proto *child;
	struct layer *l=&p->layer[p->llen];

	if ( (p->layer[p->llen+1].h.raw=
		l->h.raw+sizeof(struct pkt_ipxhdr))
		> p->end ) goto finish;

	p->llen++;

	if ( p->llen >= PKT_LAYERS ) {
		dispatch(p);
		return;
	}

	child = decode_next(l->proto, l->h.ipx->dst.sock);
	if ( child ) {
		p->layer[p->llen].flags=0;
		p->layer[p->llen].session=NULL;
		p->layer[p->llen].proto=child;
		child->decode(p);
		return;
	}

finish:
	if ( p->layer[p->llen].h.raw<p->end )
		p->layer[p->llen++].proto=NULL;
	dispatch(p);
}

static struct detect_root ipx_root;

void ipx_match(struct packet *p, int l)
{
	detect(&ipx_root, p, l);
}

int ipx_commit(struct rule *r)
{
	struct alert *a;
	struct sig_node *x;

	x = detect_new_sig(r, NULL);
	if ( x == NULL )
		return 0;

	a = calloc(1, sizeof(*a));
	if ( a == NULL ) {
		detect_free_sig(x, r->num_criteria);
		return 0;
	}

	memcpy(a, &r->alert, sizeof(*a));
	a->gen = &ipx_gen;

	return detect_add_sig(&ipx_root, x, r->num_criteria, a);
}

struct proto_table firestorm_protocols[]={
	{.proto = &ipx_p, .preq = ipx_r},
	{.proto = NULL}
};

struct plugin_desc firestorm_plugin_desc = {
	.size		= sizeof(firestorm_plugin_desc),
	.name		= "decode.ipx",
	.desc		= "Internetwork Packet eXchange",
	.author_name	= "John Leach",
	.author_email	= "john@johnleach.co.uk",
	.ver_major	= 1,
	.ver_minor	= 1,
};
