/*
 * This file is part of dotscara
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 */
#ifndef __FIBUF_HEADER_INCLUDED__
#define __FIBUF_HEADER_INCLUDED__

/* for memcpy */
#include <string.h>

/** fibuf - File input buffer object */
struct fibuf {
	/* [ Do not change these fields, they are ABI ] */
	/** File descriptor */
	int fd;
	/** EOF flag. Set to 1 if a read exposed an EOF */
	int eof;
	/** Number of valid bytes left in the buffer */
	size_t valid;
	/** virtual method table */
	const struct fibuf_ops *ops;

	/** @internal Pointer to base of buffer */
	char *buf;
	/** @internal Current read position in buffer */
	char *read_pos;
	/** @internal Total size of buffer */
	size_t buf_sz;
};

/** fibuf_ops - File input buffer virtual method table */
struct fibuf_ops {
	/** Read a 32bit integer */
	int(*read32)(struct fibuf *b, u_int32_t *val);
	/** Read a 16bit integer */
	int(*read16)(struct fibuf *b, u_int16_t *val);
	/** Read a structure of a given size */
	int(*read)(struct fibuf *b, char *data, size_t len);
};

int __public fibuf_attach(struct fibuf *b, int fd, int endian, size_t bufsz)
	__nonull(1);
void __public fibuf_detach(struct fibuf *b) __nonull(1);
void __public fibuf_close(struct fibuf *b) __nonull(1);
void __public fibuf_newfd(struct fibuf *b, int fd) __nonull(1);
static inline int fibuf_eof(struct fibuf *b) __nonull(1);
static inline int fibuf_fd(struct fibuf *b) __nonull(1);
int __public fibuf_read(struct fibuf *b, char *buf, size_t *len)
	__nonull(1,2,3);

static inline int fibuf_eof(struct fibuf *b)
{
	return ( (b->valid == 0) && (b->eof != 0) );
}

static inline int fibuf_fd(struct fibuf *b)
{
	return b->fd;
}

#endif /* __FIBUF_HEADER_INCLUDED__ */
