/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2003,2004 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
*/
#ifndef __IDX_HEADER_INCLUDED__
#define __IDX_HEADER_INCLUDED__

#include <sys/types.h>

/* --- Index on-disk format */
#define IDX_MAGIC		0x24a9cf14UL
#define IDX_NAMELEN		16

/* API key type */
#define IDX_KEY_MIN		0UL
#define IDX_KEY_MAX		0xffffffffUL
typedef u_int32_t idx_key_t;

/* Index types */
#define IDX_ORDERED_UINT8	0
#define IDX_ORDERED_UINT16	1
#define IDX_ORDERED_UINT32	2
#define IDX_ORDERED_UINT64	3
#define IDX_ORDERED_UINT128	4
#define IDX_ORDERED_INT8	5
#define IDX_ORDERED_INT16	6
#define IDX_ORDERED_INT32	7
#define IDX_ORDERED_INT64	8
#define IDX_ORDERED_INT128	9
#define IDX_SKUNK		10
#define IDX_SKUNK_NOCASE	11
#define IDX_TYPES		12
#define IDX_INVALID		0xff

/* External file offsets */
typedef u_int32_t idx_val_t;

/* Internal index file offsets */
typedef u_int32_t idx_ofs_t;

/* Hash values */
typedef u_int32_t idx_hash_t;

/* DO NOT CHANGE */
typedef u_int8_t idx_type_t;

/* First a simple header */
struct idx_hdr {
	u_int32_t magic;
	u_int32_t num;
};

/* then hdr.num of these */
struct idx_index {
	char name[IDX_NAMELEN];
	idx_type_t type;
	u_int8_t reserved1;
	u_int16_t reserved2;
	idx_ofs_t ofs;
	idx_ofs_t len;
	idx_ofs_t num_keys;
	idx_ofs_t num_vals;
};

/* ordered key node */
struct idx_onode {
	idx_key_t key;
	idx_ofs_t ofs;
	u_int32_t num;
};

/* Skunk DB structures */
#define SKUNK_HASHSZ	128UL
#define SKUNK_PRIME	16777619UL
#define SKUNK_OFFSET	2166136261UL

struct skunk_table {
	idx_ofs_t ofs;
	idx_ofs_t len;
};

struct skunk_bucket {
	idx_hash_t hash;
	idx_ofs_t kofs;
	idx_ofs_t klen;
	idx_ofs_t vofs;
	idx_ofs_t vlen;
};

/* Fowler/Noll/Vo 1A hash function.
 * Its fast to compute, its obvious code, it performs well...
 * Its here because it constitutes the on-disk-format
*/
static inline idx_hash_t skunk_hash(const char *p, size_t len)
{
	idx_hash_t h = SKUNK_OFFSET;
	const u_int8_t *data = (const u_int8_t *)p;

	while(len--)
		h = (h * SKUNK_PRIME) ^ *data++;

	return h;
}

static inline idx_hash_t skunk_hash_nocase(const char *p, size_t len)
{
	idx_hash_t h = SKUNK_OFFSET;
	const u_int8_t *data = (const u_int8_t *)p;

	while(len--)
		h = (h * SKUNK_PRIME) ^ toupper(*data++);

	return h;
}

#endif /* __IDX_HEADER_INCLUDED__ */
