/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __MATCHER_HEADER_INCLUDED__
#define __MATCHER_HEADER_INCLUDED__

#include <detect.h>

/* Matcher cost offsets */
#define MCOST_LINK	1000
#define MCOST_NET	2000
#define MCOST_TRANS	3000
#define MCOST_APP	4000
#define MCOST_DATA	5000

/* The matcher structure */
struct matcher {
	struct field *owner;

	int(*validate)(struct matcher *m, char *args,
			void **priv, struct modifier *opts);
	void(*free_priv)(void *);

	struct detect *(*new_node)(struct matcher *);
	int (*optimize)(struct detect *);
	struct detect **(*insert_node)(struct detect *d, struct sig_node *n);

	struct matcher *next;
};

/* Initialization macros */
#define matcher_null() {.name=NULL}

int matcher_add(struct matcher *m);
struct matcher *matcher_find(const char *) __nonull(1);

/* New Template API */
typedef int(*proc_template_getval)(struct packet *p, int l, idx_key_t *val)
	__nonull(1,3);
typedef int(*proc_template_getstr)(struct packet *p, int l, struct fvec *str)
	__nonull(1,3);

struct template_detect {
	struct detect d;
	struct template_node *nodes;
	proc_template_getval cbfn;
};

struct template_detect2 {
	struct detect d;
	struct template_node *nodes;
	proc_template_getstr cbfn;
};

struct template_node {
	union {
		struct {
			u_int32_t min;
			u_int32_t max;
		}val;
		struct fvec str;
	}u;
	int negate;
	struct detect *result;
	struct alert *alert;
	struct template_node *next;
};

int template_optimize(struct detect *x)
	__nonull(1);
struct detect *template_new_node(proc_template_getval cbfn)
	__nonull(1);
struct detect *template_new_node_str(proc_template_getstr cbfn)
	__nonull(1);
struct detect *template_new_node_str_nocase(proc_template_getstr cbfn)
	__nonull(1);
struct detect **template_insert_default(struct detect *d)
	__nonull(1);
struct detect **template_insert(struct detect *d, struct sig_node *n)
	__nonull(1,2);
int template_validate(struct matcher *m, char *args,
		void **priv, struct modifier *opts) __nonull(1,2,3);

#endif /* __MATCHER_HEADER_INCLUDED__ */
