/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2004 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
*/
#ifndef __NBIO_HEADER_INCLUDED__
#define __NBIO_HEADER_INCLUDED__

/* Represents a given fd */
struct nbio {
	int fd;
#define NBIO_READ	(1<<0)
#define NBIO_WRITE	(1<<1)
#define NBIO_ERROR	(1<<2)
	unsigned short mask, flags;
	void *priv;
	struct nbio_ops *ops;
	struct list_head list;
	union {
		int poll;
		void *ptr;
	}ev_priv;
};

struct nbio_ops {
	void (*read)(struct nbio *n);
	void (*write)(struct nbio *n);
	void (*dtor)(struct nbio *n);
};

/* Represents all the I/Os for a given thread (note: there is
 * no locking performed by nbio, but all functions are re-entrant.
 */
struct iothread {
	struct list_head inactive;
	struct list_head active;
	struct eventloop *plugin;
	union {
		int epoll;
		void *ptr;
	}priv;
};

/* nbio API */
void nbio_add(struct iothread *, struct nbio *) __nonull(1,2);
void nbio_del(struct iothread *, struct nbio *) __nonull(1,2);
void nbio_pump(struct iothread *) __nonull(1);
void nbio_fini(struct iothread *) __nonull(1);
int nbio_init(struct iothread *, const char *plugin) __nonull(1);
void nbio_inactive(struct iothread *, struct nbio *) __nonull(1,2);

/* eventloop plugin API */
struct eventloop {
	const char *name;
	int (*init)(struct iothread *);
	void (*fini)(struct iothread *);
	void (*pump)(struct iothread *);
	void (*inactive)(struct iothread *, struct nbio *);
	void (*active)(struct iothread *, struct nbio *);
	struct eventloop *next;
};

void eventloop_load(void);
struct eventloop *eventloop_find(const char *name) __nonull(1);

#endif /* __NBIO_HEADER_INCLUDED__ */
