/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __NOTIFIER_HEADER_INCLUDED__
#define __NOTIFIER_HEADER_INCLUDED__

#define NOTIFY_HUP		1 /* On receipt of SIGHUP */
#define NOTIFY_ITIMER		2 /* Periodic timer, may never fire */
#define NOTIFY_VTIMER		3 /* Periodic virtual timer, may never fire */
#define NOTIFY_USR1		4 /* On receipt of SIGUSR1 */
#define NOTIFY_USR2		5 /* On receipt of SIGUSR2 */
#define NOTIFY_MAX		6

/* Actually does packet preprocessing */
typedef void (*notifier_t)(struct packet *, int, void *);

/* Frees up the private data */
typedef void (*notifier_dtor_t)(void *);

struct notifier {
	void *priv;
	notifier_t fn;
	notifier_dtor_t dtor;
	struct list_head list;
};

struct notifier *notifier_add(struct list_head *, notifier_t,
				notifier_dtor_t, void *)
				__nonull(1, 2);
void notifier_delete(struct list_head *)
	__nonull(1);
void notifier_run(struct list_head *, struct packet *, int)
	__nonull(1);
void notifier_run_backwards(struct list_head *, struct packet *, int)
	__nonull(1);
void notifier_cancel(struct notifier *) __nonull(1);

struct notifier *notify(int, notifier_t, notifier_dtor_t, void *)
	__nonull(2);
void __private notify_run(int, struct packet *, int);

#endif /* __NOTIFIER_HEADER_INCLUDED__ */
