/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>
#include <alert.h>
#include <signature.h>
#include <matcher.h>
#include <decode.h>
#include <plugin.h>
#include <pkt/ipx.h>

#if 0
static int ipx_dst_match(struct packet *pkt, void *priv, int l, int n)
{
	return n^(memcmp(priv,pkt->layer[l].h.ipx->dst.node,6)==0);
}

static int ipx_src_match(struct packet *pkt, void *priv, int l, int n)
{
	return n^(memcmp(priv,pkt->layer[l].h.ipx->src.node,6)==0);
}

static int ipx_type_match(struct packet *pkt, void *priv, int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t port=pkt->layer[l].h.ipx->type;
	return n^(port>=p->min && port<=p->max);
}

static int ipx_dstsock_match(struct packet *pkt, void *priv, int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t sock=pkt->layer[l].h.ipx->dst.sock;
	return n^(sock>=p->min && sock<=p->max);
}

static int ipx_srcsock_match(struct packet *pkt, void *priv, int l, int n)
{
	struct shortrange *p=(struct shortrange *)priv;
	u_int16_t sock=pkt->layer[l].h.ipx->src.sock;
	return n^(sock>=p->min && sock<=p->max);	
}
#endif

static int ipx_addr_validate(char *args, void **priv, struct modifier *m)
{
	unsigned char buf[IPX_NODE_LEN];
	size_t i;

	if ( args == NULL )
		return 0;

	for(i=0; i<sizeof(buf);) {
		unsigned long int l;
		char *end;

		l=strtoul(args, &end, 16);
		if ( end == args )
			return 0;

		if ( l & ~0xffUL )
			return 0;

		buf[i++] == l;

		if ( *end == '\0' ) {
			if ( i != sizeof(buf) )
				return 0;
			break;
		}

		if ( *end != ':' )
			return 0;

		args = end + 1;
	}

	*priv = malloc(sizeof(buf));
	if ( *priv == NULL )
		return 0;

	memcpy(*priv, buf, sizeof(buf));
	return 1;
}

struct matcher firestorm_matchers[]={
	{
		.name = "ipx_dst",
		.desc = "IPX Destination Address",
		.validate = ipx_addr_validate,
		.free_priv = free,
		.cost = MCOST_NET,
	},
	{
		.name = "ipx_src",
		.desc = "IPX Source Address",
		.validate = ipx_addr_validate,
		.free_priv = free,
		.cost = MCOST_NET,
	},
	{
		.name = "ipxtype",
		.desc = "IPX Type",
		.validate = template_validate16,
		.insert_node = template_insert,
		//.new_node = icmp_id_new,
		.free_priv = free,
		.cost = MCOST_NET,
	},
	{
		.name = "ipxsrcsock",
		.desc = "IPX Source Socket",
		.validate = template_validate16,
		.insert_node = template_insert,
		//.new_node = icmp_id_new,
		.free_priv = free,
		.cost = MCOST_NET,
	},
	{
		.name = "ipxdstsock",
		.desc = "IPX Destination Socket",
		.validate = template_validate16,
		.insert_node = template_insert,
		//.new_node = icmp_id_new,
		.free_priv = free,
		.cost = MCOST_NET,
	},
	matcher_null()
};

struct plugin_desc firestorm_plugin_desc = {
	.size		= sizeof(firestorm_plugin_desc),
	.name		= "matcher.ipx",
	.desc		= "IPX matching routines",
	.author_name	= "John Leach",
	.author_email	= "john@johnleach.co.uk",
	.ver_major	= 1,
	.ver_minor	= 1,
};
