/******************************************************************************/
/* feedforward neural network routines                                        */
/* (c) 2003 jamie twycross, milieu3@ukshells.co.uk                            */
/* released under gnu gpl version 2                                           */
/* hacked to use doubles from lwneuralnet by peter van rossum                 */
/******************************************************************************/

#ifndef NETWORK_H
#define NETWORK_H

#include <stdio.h>

typedef struct {
	double output;
	double error;
	double *weight;
	double *delta;
} neuron_t;

typedef struct {
	unsigned int no_of_neurons;
	neuron_t *neuron;
} layer_t;

typedef struct {
	unsigned int no_of_layers;
	double momentum;
	double learning_rate;
	double global_error;
	layer_t *layer;
	layer_t *input_layer;
	layer_t *output_layer;
} network_t;

network_t *net_allocate(const unsigned int, ...);
network_t *net_allocate_l(const unsigned int, int *);
void net_free(network_t *);
network_t *net_randomize(network_t *, const double);
network_t *net_reset_deltas(network_t *);
network_t *net_set_momentum(network_t *, const double);
network_t *net_set_learning_rate(network_t *, const double);
double net_get_momentum(const network_t *);
double net_get_learning_rate(const network_t *);
unsigned int net_get_no_of_inputs(const network_t *);
unsigned int net_get_no_of_outputs(const network_t *);
unsigned int net_get_no_of_layers(const network_t *);
double net_get_weight(const network_t *, const unsigned int, \
		const unsigned int, const unsigned int);
int net_fprint(FILE *, const network_t *);
network_t *net_fscan(FILE *);
int net_print(const network_t *);
int net_save(const network_t *, const char *);
network_t *net_load(const char *);
network_t *net_compute(network_t *, const double *, double *);
double net_compute_output_error(network_t *, const double *);
double net_get_output_error(const network_t *);
network_t *net_train(network_t *);

#endif
