/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>
#include <plugin.h>
#include <args.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>

/* Training mode or not */
static unsigned int training=0;

/* Structure on disk */
struct svr_dtuple {
	u_int32_t count;
	u_int32_t addr;
	u_int16_t port;
};

/* Structure in memory */
struct svr_tuple {
	struct svr_tuple *next;
	struct svr_dtuple t;
};

#if 0
/* Allocator */
#define SPOON_ALLOC_CHUNK 8192
static struct svr_tuple *alloc_block=NULL;
static int alloc_next=SPOON_ALLOC_CHUNK;

/* Training hash table */
#define SHASH_SIZE 4096
static struct svr_tuple *shash[SHASH_SIZE];
#endif

/* Learn about packets */
static void spoon_training(struct packet *pkt, int l, void *priv)
{
	struct pkt_iphdr *iph=pkt->layer[l-1].h.ip;
	struct pkt_tcphdr *tcph;
	u_int32_t flags;
	u_int32_t addr;
	u_int16_t port;

	tcph=pkt->layer[l].h.tcp;
	flags=pkt->layer[l].flags;

	/* We want to get addresses/ports of all servers
	 * connected to during the training period */
	if ( flags & FLAG_TCP_TRACK ) {
		/* Only look at packets which signal the establishment
		 * of a successful connection. */
		struct tcp_session *s=pkt->layer[l].session;
		if ( !(flags&FLAG_TCP_CT_EST) )
			return;
		addr=s->s_addr;
		port=s->s_port;
	}else{
		/* Poor mans version, just check for tcp syn packets */
		if ( iph->protocol!=6 || (tcph->flags.flags&TCP_STD)!=TCP_SYN ) {
			return;
		}
		addr=iph->daddr;
		port=tcph->dport;
	}

	/* TODO: Add this tuple to the database */
}

static void spoon_free(void *priv){return;};

static int spoon_init(const char *args, preproc_fn fn, void *priv)
{
	struct list_head *n;
	struct arg spoon_args[]={
		{"training", ARGTYPE_PBOOL, {vp_bool:&training}},
		{NULL,}
	};

	n = (*fn)("tcp", priv);
	if ( n == NULL )
		return 0;

	if ( args ) {
		switch( args_parse(spoon_args, args, NULL) ) {
		case -1:
			mesg(M_ERR,"spoon: parse error: %s", args);
			/* fall through */
		case 0:
			return 0;
		default:
			break;
		}
	}

	if ( training ) {
		if ( !notifier_add(n, spoon_training, spoon_free, NULL) )
			return 0;
	}else{
		mesg(M_ERR,"spoon: not finished yet...");
		return 0;
	}

	return 1;
}

struct preproc firestorm_preprocs[]={
	{.name = "spoon", .init = spoon_init},
	{.name = NULL}
};

struct plugin_desc firestorm_plugin_desc = {
	.size 		= sizeof(firestorm_plugin_desc),
	.name 		= "preproc.spoon",
	.desc 		= "S.P.O.O.N. Anomaly Detection",
	.author_name 	= "Gianni Tedesco",
	.author_email 	= "gianni@scaramanga.co.uk",
	.ver_major 	= 0,
	.ver_minor 	= 1,
};
