/*
* This file is part of dotscara
* Copyright (c) 2003 Gianni Tedesco
* Released under the terms of the GNU GPL version 2
*/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/poll.h>

#include <dotscara/dotscara.h>
#include <dotscara/iowait.h>

/** __iowait_single_fd
 * @param fd: File descriptor to wait for events on
 * @param flags: Poll events to wait for
 *
 * Waits for events on a non-blocking file descriptor. This is an internal
 * API used by fibuf/fobuf etc. if the fd they are trying to use returns
 * EAGAIN, this returns the blocking behaivour.
 *
 * Return value: 1 on success, 0 on error.
 */
int __private __iowait_single_fd(int fd, int flags)
{
	struct pollfd pfd;
	int ret;

	/* Some systems don't indicate POLLIN on EOF, rather they
	 * use POLLHUP, we always check for POLLHUP then use read/
	 * write to see what the real problem is.
	 */
	flags |= POLLHUP;

	pfd.fd = fd;
	pfd.events = flags | POLLERR;
	pfd.revents = 0;

again:
	ret = poll(&pfd, 1, -1);
	if ( ret < 0 ) {
		if ( errno == EINTR )
			goto again;
		return 0;
	}

	if ( pfd.revents & flags )
		return 1;

	/* POLLERR */
	/* FIXME: Should we return 1 here and force the caller
	 * to attempt an I/O operation in order to make sure that
	 * the correct value ends up in errno?
	 */
	return 0;
}
