/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002,2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*/
#include <firestorm.h>

#define NOTIFIER_CHUNK 8

static struct list_head sysnotify[NOTIFY_MAX];

struct notifier *notify(int i, notifier_t fn, notifier_dtor_t d, void *p)
{
	if ( i >= NOTIFY_MAX || i < 0 )
		return NULL;

	return notifier_add(&sysnotify[i], fn, d, p);
}

void __private notify_run(int i, struct packet *p, int l)
{
	notifier_run(&sysnotify[i], p, l);
}

struct notifier *notifier_add(struct list_head *l, notifier_t fn,
				notifier_dtor_t d, void *p)
{
	struct notifier *n;

	n = malloc(sizeof(*n));
	if ( n == NULL )
		return NULL;

	n->priv = p;
	n->fn = fn;
	n->dtor = d;

	list_add(&n->list, l);

	return n;
}

void notifier_delete(struct list_head *l)
{
	struct notifier *n, *t;

	list_for_each_entry_safe(n, t, l, list) {
		list_del(&n->list);
		if ( n->dtor )
			n->dtor(n->priv);
		free(n);
	}
}

void notifier_run(struct list_head *l, struct packet *p, int layer)
{
	struct notifier *d;

	list_for_each_entry(d, l, list)
		d->fn(p, layer, d->priv);
}

void notifier_cancel(struct notifier *n)
{
	list_del(&n->list);
	if ( n->dtor )
		n->dtor(n->priv);
	free(n);
}

static void notifier_cleanup(struct packet *pkt, int l, void *priv)
{
	int i;

	for(i=0; i< NOTIFY_MAX; i++)
		notifier_delete(&sysnotify[i]);
}

void __private notifier_init(void)
{
	int i;

	for(i=0; i< NOTIFY_MAX; i++)
		INIT_LIST_HEAD(&sysnotify[i]);

	notify_exit(notifier_cleanup, NULL);
}
