/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002,2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
* This is the preprocessor subsystem. We keep track of all
* registered preprocessors and we also manage the instanciation
* and execution of preprocessors.
*
* Only preprocessors specified in the configuration are actually
* used. preproc_setup() is called once for each preprocessor
* that is specified. The plugin then calls preproc_activate() if
* it wants to recieve notification of all packets of a given
* protocol.
*/
#include <firestorm.h>
#include <args.h>
#include <loader.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>

/* All registered preprocessors */
static struct preproc *preprocs;

/* Find a preprocessor with a given name */
struct preproc *preproc_find(const char *n)
{
	struct preproc *p;

	for(p=preprocs; p; p=p->next) {
		if ( !strcmp(p->name, n) )
			return p;
	}

	return NULL;
}

static void preproc_add(struct plugin *p, struct preproc *o)
{
	for(; o->name; o++) {
		if ( preproc_find(o->name) )
			continue;

		o->next = preprocs;
		preprocs = o;
	}
}

void preproc_load(void)
{
	loader_find_syms("firestorm_preprocs", (loader_cb_t)preproc_add);
}
