/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco
 * This program is released under the terms of the GNU GPL version 2
*/

#include <firestorm.h>
#include <strtouint.h>
#include <limits.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

int strtouint(const char *s, unsigned int *v)
{
	unsigned long int ret;
	char *end=NULL;

	ret=strtoul(s, &end, 0);

	if ( end==s )
		return -1;

	if ( ret == ULONG_MAX && errno==ERANGE )
		return -1;

	if ( ret > UINT_MAX )
		return -1;

	*v = ret;

	if ( *end == '\0' )
		return 0;

	return end - s;
}

int strtoip(const char *s, u_int32_t *ip)
{
	struct in_addr tmp;

	if ( !inet_aton(s, &tmp) ) {
		*ip = 0;
		return 0;
	}

	*ip = tmp.s_addr;
	return 1;
}

const char *iptostr(u_int32_t ip)
{
	struct in_addr in;
	in.s_addr = ip;
	return inet_ntoa(in);
}
